///////////////////////////////////////////////////////////////////////////////
//
// StlAllocatorTestOutput.h
//
// Copyright  2003 Pete Isensee (PKIsensee@msn.com).
// All rights reserved worldwide.
//
// Permission to copy, modify, reproduce or redistribute this source code is
// granted provided the above copyright notice is retained in the resulting 
// source code.
// 
// This software is provided "as is" and without any express or implied
// warranties.
//
// Description:
//
//    Error and output functions

#if !defined( STL_ALLOCATOR_TEST_OUTPUT_H )
#define STL_ALLOCATOR_TEST_OUTPUT_H
#pragma once

#include <cstdio>
#include <cstdarg>
#include <iostream>

#if defined( _XBOX )
    #include "xtl.h"
#else if defined( WIN32 )
    #include "windows.h"    
#endif    

//-----------------------------------------------------------------------------

// Forces the compiler to believe that 'a' cannot be optimized away
#define USED(a) Used( 0, a )

// Verifies the expression; also forces the compiler to believe that 'b'
// cannot be optimized away.
#define VERIFY(b) (void)( (USED(b),b) || (OutputError( #b, __FILE__, __LINE__ ), 0) )

// Output error message; includes file and line number
#define OUTERR(e) OutputError( e, __FILE__, __LINE__ )

namespace StlAllocatorTestbed
{

//-----------------------------------------------------------------------------

// This function forces the compiler to believe that the optional parameters
// are important and cannot be optimized away
void Used( int i, ... )
{
    va_list Marker;
    va_start( Marker, i );
    i = va_arg( Marker, int );
    va_end( Marker );
}

//-----------------------------------------------------------------------------

void OutputError( const char* Expression, const char* File, unsigned int Line )
{
#if defined( _XBOX )
    std::ostringstream s;
    s << File << "(" << Line << ") : error: " << Expression << std::endl; 
    OutputDebugString( s.str().c_str() );
    DebugBreak();
#else    
    // Match the format of the Visual C++ compiler error messages
    std::cerr << File << "(" << Line << ") : error: " << Expression << std::endl;
    #if defined( WIN32 )
        DebugBreak();    
    #endif    
#endif    
}

} // end StlAllocatorTestbed

#endif // STL_ALLOCATOR_TEST_OUTPUT_H
